# Copyright (c) 2018 Simon Guest
#
# This file is part of public-wrappers.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of
# this software and associated documentation files (the "Software"), to deal in
# the Software without restriction, including without limitation the rights to
# use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
# the Software, and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
# FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
# COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
# IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

from future.utils import viewitems
from past.builtins import basestring
import os.path
import pytoml as toml

from .ConfigError import ConfigError

def expand(s):
    return os.path.expanduser(os.path.expandvars(s))

class ConfigObject:
    def __init__(self, filename, context, obj):
        self.filename = filename
        self.context = context
        self.obj = obj

    def error(self, msg):
        raise ConfigError(self.filename, self.context, msg)

    def iter(self, key):
        """Iterate over values for key, which is allowed to not exist."""
        if key in self.obj:
            for subkey, subobj in viewitems(self.obj[key]):
                yield subkey, ConfigObject(self.filename, self.context + [key, subkey], subobj)

    def getConfigOrDie(self, key):
        """Return config at key, or raise a ConfigError."""
        if key in self.obj:
            return ConfigObject(self.filename, self.context + [key], self.obj[key])
        else:
            self.error('missing %s' % key)

    def _getValue(self, key):
        value = self.obj[key]
        if isinstance(value, basestring):
            return expand(value)
        else:
            return value

    def getValueOrDie(self, key):
        """Return value at key, or raise a ConfigError."""
        if key in self.obj:
            return self._getValue(key)
        else:
            self.error('missing %s' % key)

    def getValueOrNone(self, key):
        """Return value at key, or None."""
        if key in self.obj:
            return self._getValue(key)
        else:
            return None

def read_config_file(args):
    filename = None
    if args.config:
        filename = args.config
        if not os.path.exists(filename):
            raise ConfigError('%s' % filename, [], 'file not found')
    else:
        env_config = 'PUBLIC_WRAPPERS_CONFIG'
        attempt_files = ([os.environ[env_config]] if env_config in os.environ else []) + [
            os.path.expanduser('~/.public-wrappers.toml'),
            '/etc/public-wrappers.toml',
        ]
        for attempt in attempt_files:
            if os.path.exists(attempt):
                filename = attempt
                break
        if filename is None:
            raise ConfigError('', [], 'none of %s found' % ', '.join(attempt_files))

    with open(filename, 'rb') as f:
        try:
            obj = toml.load(f)
        except toml.TomlError as e:
            raise ConfigError(filename, [], 'TOML error at line %d' % e.line)
    return ConfigObject(filename, [], obj)



#!/usr/bin/env python
#
# distutils setup script for public-wrappers package

from setuptools import setup, find_packages

long_description = """public-wrappers provides a means to wrap selected public programs from conda
environments, so they may be invoked directly by users without having to
explicitly activate the environment.  It makes use of exec-wrappers to do the
wrapping.
"""

setup(name='public-wrappers',
      use_scm_version=True,
      setup_requires=['setuptools_scm', 'setuptools_scm_git_archive'],
      description='Wrap selected public programs from conda/virtualenv environments',
      long_description=long_description,
      author='Simon Guest',
      author_email='simon.guest@tesujimath.org',
      url='https://github.com/tesujimath/public-wrappers',
      classifiers=[
          'Development Status :: 4 - Beta',
          'Environment :: Console',
          'Intended Audience :: System Administrators',
          'License :: OSI Approved :: MIT License',
          'Operating System :: POSIX :: Linux',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3',
          'Topic :: System :: Systems Administration',
          'Topic :: Utilities',
      ],
      packages=find_packages(),
      entry_points={
        'console_scripts': [
            'configure-public-wrappers = public_wrappers.__main__:main',
        ],
      },
      package_data={
          'doc': ['*.rst'],
      },
      license='MIT',
      install_requires=[
          'exec-wrappers',
          'future',
          'pytoml',
      ],
      python_requires='>=2.7',
     )
